import argparse
import numpy as np
import cv2

# http://nghiaho.com/?page_id=846
# http://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html?highlight=findhomography

# Decompoem uma matriz de projeção em Matriz de Câmera e Matriz de Rotação
def decomposeWarpMatrix(H):
	# Cria matriz de projeção com última coluna (Translação) = 0
	projMat = np.zeros((3,4))
	projMat[:,:3] = H

	P = cv2.decomposeProjectionMatrix(projMat)
	(A,R) = P[:2]

	return (A,R)

# Retorna os Ângulos X, Y e Z que compoem a matriz
def decomposeRotationMatrix(R):
	degX = np.arctan2(R[2,1], R[2,2])
	degY = np.arctan2(-R[2,0], np.sqrt(R[2,1]**2 + R[2,2]**2))
	degZ = np.arctan2(R[1,0], R[0,0])

	return (degX, degY, degZ)

# Cria matriz de rotação com base nos ângulos de rotação em X, Y e Z
def composeRotationMatrix(x,y,z):
	X = np.zeros((3,3))
	Y = np.zeros((3,3))
	Z = np.zeros((3,3))

	# X
	X[0,0] = 1
	X[1,1] = np.cos(x)
	X[1,2] = -np.sin(x)
	X[2,1] = np.sin(x)
	X[2,2] = np.cos(x)
	#print(X)

	# Y
	Y[1,1] = 1
	Y[0,0] = np.cos(y)
	Y[0,2] = np.sin(y)
	Y[2,0] = -np.sin(y)
	Y[2,2] = np.cos(y)
	#print(Y)

	# Z
	Z[2,2] = 1
	Z[0,0] = np.cos(z)
	Z[0,1] = -np.sin(z)
	Z[1,0] = np.sin(z)
	Z[1,1] = np.cos(z)
	#print(Z)

	# Z * Y * X
	return np.dot(np.dot(Z, Y), X)

def composeWarpMatrix(x,y,z,A):
	R = composeRotationMatrix(x,y,z)
	return np.dot(A,R)

def composeCameraMatrix(fx, fy, sh_d, cx, cy):
	A = np.zeros((3,3))
	A[0,0] = fx
	A[1,1] = fy
	A[2,2] = 1
	A[0,2] = cx
	A[1,2] = cy

	'''
	if sh_d == np.pi / 2:
		sh = 0
	else:
		tg = np.tan(sh_d)
		if tg == 0:
			# undefined
			print("tg undefined")
			sh = 0
		else:
			sh = 1 / tg
	'''
	A[0,1] = sh_d

	return A

if __name__ == '__main__':
	ap = argparse.ArgumentParser()
	ap.add_argument("-m", "--matrix", required=True,
		help="path to h matrix")
	args = vars(ap.parse_args())

	matH = np.loadtxt(args["matrix"])

	# Decompoem Matriz H para encontrar Matriz da Câmera e ângulos de rotação
	(A,R) = decomposeWarpMatrix(matH)
	degs = decomposeRotationMatrix(R) # ângulos
	(x,y,z) = degs
	print(degs)

	np.savetxt("matA.txt", A)
	np.savetxt("angulos.txt", degs)
	np.savetxt("matX.txt", composeRotationMatrix(x,0,0))
	np.savetxt("matY.txt", composeRotationMatrix(0,y,0))
	np.savetxt("matZ.txt", composeRotationMatrix(0,0,z))


	# Remonta Matriz H para comparação
	compMat = np.dot(A, composeRotationMatrix(x,y,z))

	np.savetxt("matHnova.txt", compMat)
