import numpy as np
import argparse
import cv2
import imutils
import imageio

from decompose_warp import composeWarpMatrix, composeCameraMatrix, decomposeWarpMatrix, decomposeRotationMatrix
from apply_h import applyTransformationMatrix

def linearInterpolation(begin,end,steps):
	iter_step = (end - begin) / steps
	cur_step = 0

	while cur_step < steps:
		yield begin + iter_step * cur_step
		cur_step += 1

	# Prevenir contra erros de arredondamento: Último passo sempre retorna end
	yield end

def linearInterpolateFramesH(img, H, steps=2, defaultBorder=[0,0,0]):
	(A, R) = decomposeWarpMatrix(H)
	(x, y, z) = decomposeRotationMatrix(R)

	return linearInterpolateFrames(img, x, y, z, A, steps, defaultBorder)

def linearInterpolateFrames(img, x, y, z, A, steps=2, defaultBorder=[0,0,0]):
	fx = A[0,0]
	fy = A[1,1]
	sh = A[0,1]
	[cx,cy] = A[:2,2]

	# Valor de shearing é um cotg
	#if sh == 0:
		#sh = np.pi/2
	#else:
		#sh = np.arctan(1 / sh)

	H0 = composeWarpMatrix(0,0,0,np.identity(3))
	im0 = applyTransformationMatrix(img,H0)

	iterX = linearInterpolation(0,x,steps)
	iterY = linearInterpolation(0,y,steps)
	iterZ = linearInterpolation(0,z,steps)

	iterFx = linearInterpolation(1,fx,steps)
	iterFy = linearInterpolation(1,fy,steps)
	#iterSh = linearInterpolation(np.pi/2,sh,steps)
	iterSh = linearInterpolation(0,sh,steps)
	iterCx = linearInterpolation(0,cx,steps)
	iterCy = linearInterpolation(0,cy,steps)

	frames = []

	for i in range(steps+1):
		A = composeCameraMatrix(next(iterFx), next(iterFy), next(iterSh), next(iterCx), next(iterCy))
		H = composeWarpMatrix(next(iterX), next(iterY), next(iterZ), A)
		trans_image = applyTransformationMatrix(img,H,defaultBorder=[255,255,255])#[48,255,0])
		frames.append(trans_image[:,:,::-1]) # Inverte ordem das cores pois o CV2 guardava BGR e o Imageio usa RGB

	return frames
	#imageio.mimsave("gif.gif", frames)

if __name__ == '__main__':
	ap = argparse.ArgumentParser()

	ap.add_argument("-d", "--degrees",
		help="arquivo com os angulos")
	ap.add_argument("-a", "--matrixA",
		help="arquivo com a matriz A")
	ap.add_argument("-i", "--image",
		help="arquivo da imagem")
	ap.add_argument("-s", "--steps", type=int,
		help="iteracoes ate o final")
	args = vars(ap.parse_args())

	file_degrees = args["degrees"] if args["degrees"] is not None else "angulos.txt"
	file_A = args["matrixA"] if args["matrixA"] is not None else "matA.txt"
	file_image = args["image"] if args["image"] is not None else "030-cut.png"
	steps = args["steps"] if args["steps"] is not None else 2

	degs = np.loadtxt(file_degrees)
	Af = np.loadtxt(file_A) # Matriz da Câmera
	img = imutils.resize(cv2.imread(file_image), width=400)

	print(degs)

	[x,y,z] = degs

	frames = linearInterpolateFrames(img, x, y, z, Af)

	imageio.mimsave("gif.gif", frames)