import numpy as np
import imutils
import cv2
import argparse
import matplotlib.pyplot as plt

# http://docs.opencv.org/3.0-beta/doc/py_tutorials/py_gui/py_image_display/py_image_display.html

class Stitcher:
	def __init__(self):
		# determine if we ar eusing OpenCV v3.x
		self.isv3 = imutils.is_cv3()

	#def npToCv2(self, image):
		#h,w = image.shape
		#vis = cv2.CreateMat(h, w, cv2.CV_32FC3)

	def calcDif(self, imA, imB):
		#heat = np.zeros(imB.shape, np.uint8)
		grayA = cv2.cvtColor(imA[0:imB.shape[0], 0:imB.shape[1]], cv2.COLOR_BGR2GRAY)
		grayB = cv2.cvtColor(imB, cv2.COLOR_BGR2GRAY)
		heat = np.zeros(grayB.shape, np.int16)
		heat[:] = grayB
		heat[:] = heat - grayA
		heat[heat < 0] = heat[heat < 0] * -1
		return cv2.applyColorMap(heat.astype(np.uint8), cv2.COLORMAP_HOT)

	def stitch(self, images, ratio=0.75, reprojThresh=4.0,
		showMatches=False):
		# unpack the images, then detect keypoints and extract
		# local invariant descriptors from them
		# Invariant = Invariante à rotação e escala
		# Imagens estão na ordem left-to-right
		(imageB, imageA) = images
		(kpsA, featuresA) = self.detectAndDescribe(imageA)
		(kpsB, featuresB) = self.detectAndDescribe(imageB)

		kpsA_float = np.float32([kp.pt for kp in kpsA])
		kpsB_float = np.float32([kp.pt for kp in kpsB])

		# match features between two images
		M = self.matchKeypoints(kpsA_float, kpsB_float,
			featuresA, featuresB, ratio, reprojThresh)

		# if the Match is None, then there aren't enough matched
		# keypoints to create a panorama

		if M is None:
			return None

		# otherwise, apply a perspective warp to stitch the images
		# together
		(matches, H, status) = M
		result = cv2.warpPerspective(imageA, H,
			(imageA.shape[1] + imageB.shape[1], max(imageA.shape[0],
				imageB.shape[0]) * 2))
		cv2.imshow("asd", result)

		heat = self.calcDif(result, imageB)
		cv2.imshow("Heat", heat)

		result[0:imageB.shape[0], 0:imageB.shape[1]] = imageB
		#result[0:imageB.shape[0], 0:imageB.shape[1]] = (result[0:imageB.shape[0], 0:imageB.shape[1]] + imageB) / 2
		print(H)

		np.savetxt("matH.txt", H)

		H2 = np.zeros((3,3))
		H2[0:2,:] = H[0:2,:] / 2
		H2[2,0:2] = H[2,0:2] / 2
		H2[2,2] = 1
		cv2.imwrite("asdinter.png", cv2.warpPerspective(imageA, H2,
			(imageA.shape[1], imageA.shape[0])))

		# checkto see if the keypoint matches should be visualized
		if showMatches:
			vis = self.drawMatches(cv2.drawKeypoints(cv2.cvtColor(imageA,cv2.COLOR_BGR2GRAY),kpsA,imageA,flags=cv2.DRAW_MATCHES_FLAGS_DRAW_RICH_KEYPOINTS), 
								   cv2.drawKeypoints(cv2.cvtColor(imageB,cv2.COLOR_BGR2GRAY),kpsB,imageB,flags=cv2.DRAW_MATCHES_FLAGS_DRAW_RICH_KEYPOINTS)
				, kpsA_float, kpsB_float, matches,
				status)

			# return a tuple of the stitched image and the
			# visualization
			return (result, H, vis)

		return (result, H)

	def detectAndDescribe(self, image):
		# convert the image to grayscale
		gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
		cv2.imshow("Gray", gray)

		# check to see if we are using cv 3.x
		if self.isv3:
			# detect and extract features from the image
			descriptor = cv2.xfeatures2d.SIFT_create()
			(kps, features) = descriptor.detectAndCompute(image, None)

		# OpenCV 2.4x
		else:
			# detect keypoints in the image
			detector = cv2.FeatureDetector_create("SIFT")
			kps = detector.detect(gray)

			# extract features from the image
			extractor = cv2.DescriptorExtractor_create("SIFT")
			(kps, features) = extractor.compute(gray, kps)

		# convert the keypoints from KeyPoint objects to np arrays
		#kps = np.float32([kp.pt for kp in kps])

		# return a tuple of keypoints and features
		return (kps, features)

	def matchKeypoints(self, kpsA, kpsB, featuresA, featuresB,
		ratio, reprojThresh):
		# compute the raw matches and initialize the list of actual
		# matches
		matcher = cv2.DescriptorMatcher_create("BruteForce")
		rawMatches = matcher.knnMatch(featuresA, featuresB, 2)
		matches = []

		# loop over the raw matches
		for m in rawMatches:
			# ensure the distance is withing a certain ratio of
			# each other (i.e. Lowe's ratio test)
			if len(m) == 2 and m[0].distance < m[1].distance * ratio:
				matches.append((m[0].trainIdx, m[0].queryIdx))

		# computing a homography requires at least 4 matches
		if len(matches) > 4:
			# construct the two sets of points
			ptsA = np.float32([kpsA[i] for (_, i) in matches])
			ptsB = np.float32([kpsB[i] for (i, _) in matches])

			# compute the homography between the two sets of points
			(H, status) = cv2.findHomography(ptsA, ptsB, cv2.RANSAC,
				reprojThresh)

			# return the matches along with the homography matrix
			# and status of each matches point
			return (matches, H, status)

	def drawMatches(self, imageA, imageB, kpsA, kpsB, matches, status):
		# initialize the output visualization image
		(hA, wA) = imageA.shape[:2]
		(hB, wB) = imageB.shape[:2]
		# Cria 0s com largura de A + B, altura do maior, e 3 dimensões
		vis = np.zeros((max(hA, hB), wA + wB, 3), dtype="uint8")
		vis[0:hA, 0:wA] = imageA
		vis[0:hB, wA:] = imageB

		# loop over the matches
		for ((trainIdx, queryIdx), s) in zip(matches, status):
			# only process the match if the keypoint was successfully
			# matched
			if s == 1:
				# draw the match
				ptA = (int(kpsA[queryIdx][0]), int(kpsA[queryIdx][1]))
				ptB = (int(kpsB[queryIdx][0]) + wA, int(kpsB[queryIdx][1]))
				cv2.line(vis, ptA, ptB, (255, 0, 0), 1)

		# return the visualization
		return vis

if __name__ == '__main__':
	ap = argparse.ArgumentParser()
	ap.add_argument("-f", "--first", required=True,
		help="path to the first image")
	ap.add_argument("-s", "--second", required=True,
		help="path to the second image")
	args = vars(ap.parse_args())

	# load the two images and resize them to have a width of 400 pixels
	# (for faster processing)
	imageA = cv2.imread(args["second"])
	imageB = cv2.imread(args["first"])

	#cv2.imwrite(args["second"] + "-gray", cv2.cvtColor(imageA, cv2.COLOR_BGR2GRAY))
	#cv2.imwrite(args["first"] + "-gray", cv2.cvtColor(imageB, cv2.COLOR_BGR2GRAY))

	imageA = imutils.resize(imageA, width=400)
	imageB = imutils.resize(imageB, width=400)

	# stitch the images together to create a panorama
	stitcher = Stitcher()
	res = stitcher.stitch([imageA, imageB], showMatches=True)

	if res is not None:
		(result, H, vis) = res

		# show the images
		cv2.imshow("Image A", imageA)
		cv2.imshow("Image B", imageB)
		cv2.imshow("Keypoint Matches", vis)
		cv2.imshow("Result", result)
		cv2.imwrite("vis.png", vis)

	cv2.waitKey(0)